<?php

/**
 * An object containing all the data and member functions to provide view content to a widget.
 */
class embed_widget_view extends embed_widget {

  protected $view = NULL;

  function __construct($type = NULL, $id = NULL, $delta = NULL, $platform = 'html', $format = 'iframe', $cache_id = NULL) {
    parent::__construct($type, $id, $delta, $platform, $format, $cache_id);
    if (module_exists('views') && empty($this->view)) {
      $this->view = views_get_view($this->source_id);
      $width = $this->view->display[$this->source_delta]->display_options['width'];
      $height = $this->view->display[$this->source_delta]->display_options['height'];
      if (!empty($width) && is_numeric($width)) {
        $this->set_variable('width', $width);
      }
      if (!empty($height) && is_numeric($height)) {
        $this->set_variable('height', $height);
      }
    }

  }

  static public function info() {
    if (module_exists('views')) {
      return array(
        'title' => t('View'),
        'description' => t('Allows Views to be used in embed widgets.'),
        'name' => 'view',
      );
    }
  }

  protected function load_content() {
    if (module_exists('views')) {
      if (empty($this->view)) {
        $this->view = views_get_view($this->source_id);
        if ($this->view == NULL) {
          return FALSE;
        }
      }
      return TRUE;
    }
  }

  protected function view_content() {
    if (module_exists('views')) {
      $view = $this->view;
      if (!empty($view)) {
        $args = is_array($this->params) ? $this->params : array();
        return $view->preview($this->source_delta, $args);
      }
      return t('The view could not be loaded.');
    }
    drupal_set_message(t('Views module must be enabled to display views in widgets.'), 'warning');
    return t('Error loading content.');;
  }

  protected function get_title() {
    $title = $this->view->get_title();
    if (empty($title)) {
      $title = $this->view->display['default']->display_options['title'];
    }
    if (empty($title)) {
      return FALSE;
    }
    return $title;
  }

  public function check_access() {
    if (!module_exists('views')) {
      return FALSE;
    }
    $view = $this->view;
    return $view->access($this->source_delta);
  }

  static public function path_load() {
    $item = menu_get_item();
    if ($item['page_callback'] == 'views_page') {
      return embed_widgets_new('view', $item['page_arguments'][0]);
    }
    return NULL;
  }

}
