<?php
/**
 * @file
 * Implementation of widget display.
 */

/**
 * The plugin that handles a feed, such as RSS or atom.
 *
 * For the most part, feeds are page displays but with some subtle differences.
 */
class embed_widgets_plugin_display_embed_widgets extends views_plugin_display_attachment {

  function get_style_type() {
    return 'widget';
  }

  /**
   * web_widgets do not go through the normal page theming mechanism. Instead, they
   * go through their own little theme function and then return NULL so that
   * Drupal believes that the page has already rendered itself...which it has.
   */
  function preview() {
    // TODO: Render widget with no javascript
    return $this->view->render();
  }
  
  /**
   * Provide the default form for setting options.
   */
  function options_form(&$form, &$form_state) {
    // It is very important to call the parent function here.
    parent::options_form($form, $form_state);

    switch ($form_state['section']) {
      case 'dimension':
        $form['width'] = array(
          '#title' => t('Width'),
          '#type' => 'textfield',
          '#description' => t('The width of the widget in pixels.'),
          '#default_value' => $this->get_option('width'),
        );
        $form['height'] = array(
          '#title' => t('Height'),
          '#type' => 'textfield',
          '#description' => t('The height of the widget in pixels.'),
          '#default_value' => $this->get_option('height'),
        );
        break;
    }
  }

  /**
   * Perform any necessary changes to the form values prior to storage.
   * There is no need for this function to actually store the data.
   */
  function options_submit($form, &$form_state) {
    // It is very important to call the parent function here:
    parent::options_submit($form, $form_state);
    switch ($form_state['section']) {
      case 'dimension':
        $this->set_option('width', $form_state['values']['width']);
        $this->set_option('height', $form_state['values']['height']);       
        break;
    }
  }

  /**
   * Provide the summary for page options in the views UI.
   *
   * This output is returned as an array.
   */
  function options_summary(&$categories, &$options) {
    // It is very important to call the parent function here:
    parent::options_summary($categories, $options);

    unset($options['inherit_exposed_filters']);

    $categories['widget'] = array(
      'title' => t('Widget settings'),
      'data' => array(),
    );

    $x = $this->get_option('width');
    $y = $this->get_option('height');
    if (!empty($x) && !empty($y)) {
      $dim = t("X: !x Y: !y", array('!x' => $x, '!y' => $y));
    }
    else {
      $dim = t('Not set yet');
    }
    $options['dimension'] = array(
      'category' => 'page',
      'title' => t('Widget dimensions'),
      'value' => $dim,
    );
  }

  /**
   * Attach to another view.
   */
  function attach_to($display_id) {
    $displays = $this->get_option('displays');

    if (empty($displays[$display_id])) {
      return;
    }

    if (!$this->access()) {
      return;
    }

    $args = $this->get_option('inherit_arguments') ? $this->view->args : array();

    // because of this, it is very very important that displays that can accept
    // attachments not also be attachments, or this could explode messily.
    global $widgets_mode;
    if (empty($widgets_mode)) {
      $widget = embed_widgets_load('view', $this->view->name, $this->display->id);
      $widget->params = $args;
      $attachment = embed_widgets_share($widget);

    }

    switch ($this->get_option('attachment_position')) {
      case 'before':
        $this->view->attachment_before .= $attachment;
        break;
      case 'after':
        $this->view->attachment_after .= $attachment;
        break;
      case 'both':
        $this->view->attachment_before .= $attachment;
        $this->view->attachment_after .= $attachment;
        break;
    }

    $width = $this->get_option('width');
    $height = $this->get_option('height');
  }

  function uses_exposed() {
    return FALSE;
  }

  function displays_exposed() {
    return TRUE;
  }

  /**
   * Can this display accept attachments?
   */
  function accept_attachments() {
    return TRUE;
  }

  /**
   * Add this display's path information to Drupal's menu system.
   */
  function execute_hook_menu() {
    return array();
  }

}
